unit TitlePanel;
{ TitlePanel Pack
  Version 1.7 ( August 2000)

  Copyright (c) 2000, Petter Holmstrm. All Rights Reserved.

  This is a package of visual components.
  It may be used freely in any application. 
  This software is distributed "AS-IS" without any warranties.

  Components:

  TTitlePanel:             Ms-Outlook- or InstallShield Express 3.0- like page-
                           header with icon and text.
  TSectionTitlePanel:      InstallShield Express 3.0-like section-header with
                           bitmap and text.
  TSectionTitleBevel:      Corel-like dialog group header.
  TDialogTitlePanel:       Windows 2000-like dialog header with icon, title and
                           description.

  If you like this software, please notify me.

  My e-mail address: petter.holmstrom@parnet.fi
  My home page: http://www.parnet.fi/~pkh/

  ------------------------------------------------------------------------------

  Microsoft, Outlook, Windows and Internet Explorer are trademarks or
  registered trademarks of Microsoft Corporation.

  InstallSheild is a registered trademark and servicemark
  of InstallShield Corporation.

  Corel is a registered trademark of Corel Corporation.
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, Buttons;

type
  TIconType = (itCustom, itForm, itApplication);
  TIconAlign = (iaLeft, iaRight);

  TCustomTitlePanel = class(TGraphicControl)
  private
    EIcon: TIcon;
    EBorder: boolean;
    EIconType: TIconType;
    ERecord: string;
    ERecFont: TFont;
    EIconAlign: TIconAlign;
    procedure SetIcon(Value: TIcon);
    procedure SetBorder(Value: boolean);
    procedure SetIconAlign(Value: TIconAlign);
    procedure SetIconType(Value: TIconType);
    procedure SetRecord(Value: string);
    procedure SetRecFont(Value: TFont);
    procedure IconChanged(Sender: TObject);
  protected
    procedure Paint; override;
    procedure CMTextChanged(var Msg: TMessage); message CM_TEXTCHANGED;
    procedure CMFontChanged(var Msg: TMessage); message CM_FONTCHANGED;
    procedure CMColorChanged(var Msg: TMessage); message CM_COLORCHANGED;
    property Border: boolean read EBorder write SetBorder;
    property Icon: TIcon read EIcon write SetIcon;
    property IconAlign: TIconAlign read EIconAlign write SetIconAlign;
    property IconType: TIconType read EIconType write SetIconType;
    property RecordText: string read ERecord write SetRecord;
    property RecordFont: TFont read ERecFont write SetRecFont;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
  end;

  TTitlePanel = class(TCustomTitlePanel)
  published
    property Align;
    property Anchors;
    property Border;
    property Caption;
    property Constraints;
    property Color;
    property Enabled;
    property Font;
    property Icon;
    property IconAlign;
    property IconType;
    property ParentFont;
    property RecordFont;
    property RecordText;
    property Visible;
  end;

  TSectionTitlePanel = class(TCustomTitlePanel)
  private
    EBitmap: TBitmap;
    procedure BitmapChanged(Sender: TObject);
    procedure SetBitmap(Value: TBitmap);
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Align;
    property Anchors;
    property Bitmap: TBitmap read EBitmap write SetBitmap;
    property Border;
    property Caption;
    property Constraints;
    property Color;
    property Enabled;
    property Font;
    property ParentFont;
    property Visible;
  end;

  TSectionTitleBevel = class(TCustomTitlePanel)
  protected
    EIndent: integer;
    procedure Paint; override;
    procedure SetIndent(Value: integer);
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Align;
    property Anchors;
    property Caption;
    property Constraints;
    property Color;
    property Enabled;
    property Font;
    property Indent: integer read EIndent write SetIndent;
    property ParentFont;
    property Visible;
  end;

  TDialogTitlePanel = class(TCustomTitlePanel)
  private
    ETitle: string;
    ETitleFont: TFont;
    ETitleTop: integer;
    ETitleLeft: integer;
    ECaptionTop: integer;
    ECaptionLeft: integer;
    procedure SetTitleFont(Value: TFont);
    procedure SetTitle(Value: string);
    procedure SetTitleLeft(Value: integer);
    procedure SetTitleTop(Value: integer);
    procedure SetCaptionLeft(Value: integer);
    procedure SetCaptionTop(Value: integer);
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Align;
    property Anchors;
    property Caption;
    property CaptionLeft: integer read ECaptionLeft write SetCaptionLeft;
    property CaptionTop: integer read ECaptionTop write SetCaptionTop;
    property Constraints;
    property Color;
    property Enabled;
    property Font;
    property Icon;
    property IconType;
    property ParentFont;
    property Title: string read ETitle write SetTitle;
    property TitleLeft: integer read ETitleLeft write SetTitleLeft;
    property TitleTop: integer read ETitleTop write SetTitleTop;
    property TitleFont: TFont read ETitleFont write SetTitleFont;
    property Visible;
  end;

procedure Register;

implementation

{ TCustomTitlePanel }

constructor TCustomTitlePanel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ERecFont := TFont.Create;
  EIcon := TIcon.Create;
  EIcon.OnChange := IconChanged;
  EIconAlign := iaLeft;
  EBorder := True;
  ControlStyle := ControlStyle + [csOpaque];
  Color := clGray;
  Width := 200;
  Height := 38;
end;

destructor TCustomTitlePanel.Destroy;
begin
  EIcon.Free;
  ERecFont.Free;
  inherited;
end;

procedure TCustomTitlePanel.CMTextChanged(var Msg: TMessage);
begin
  inherited;
  invalidate;
end;

procedure TCustomTitlePanel.CMFontChanged(var Msg: TMessage);
begin
  inherited;
  invalidate;
end;

procedure TCustomTitlePanel.CMColorChanged(var Msg: TMessage);
begin
  inherited;
  invalidate;
end;

procedure TCustomTitlePanel.Paint;
var
  R, BrdR1, BrdR2: TRect;
  y: Integer;
begin
  R := ClientRect;
  Canvas.Brush.Color := Self.Color;
  Canvas.FillRect(R);
  if Border then
  begin
    BrdR1 := ClientRect;
    BrdR2 := ClientRect;

    with BrdR2 do
    begin
      Left   := BrdR1.Left +1;
      Top    := BrdR1.Top +1;
      Right  := BrdR1.Right -1;
      Bottom := BrdR1.Bottom -1;
    end;

    Frame3D(Canvas, BrdR1, GetSysColor(COLOR_3DSHADOW), GetSysColor(COLOR_3DHILIGHT), 1);
    Frame3D(Canvas, BrdR2, GetSysColor(COLOR_3DDKSHADOW), GetSysColor(COLOR_3DLIGHT), 1);
  end;
  if not EIcon.Empty then
  begin
    y := (Clientheight - EIcon.height) div 2;
    if EIconAlign = iaLeft then
    begin
      Canvas.Draw(R.left +4, y, EIcon);
      R.Left := R.Left + EIcon.Width;
    end
    else if EIconAlign = iaRight then
    begin
      Canvas.Draw(R.Right - 4 - EIcon.Width, y, EIcon);
      R.Right := R.Right - EIcon.Width;
    end;
  end;
  R.Left := R.Left + 8;
  R.Right := R.Right - 8;
  Canvas.Font.Assign(Self.Font);
  Canvas.Brush.Style := bsClear;
  DrawText(Canvas.Handle, PChar(Caption), Length(Caption), R, DT_SINGLELINE or DT_VCENTER);

  Canvas.Font.Assign(Self.RecordFont);
  Canvas.Brush.Style := bsClear;
  DrawText(Canvas.Handle, PChar(RecordText), Length(RecordText), R, DT_SINGLELINE or DT_VCENTER or DT_RIGHT);
end;

procedure TCustomTitlePanel.IconChanged(Sender: TObject);
begin
  Invalidate;
end;

procedure TCustomTitlePanel.SetIcon(Value: TIcon);
begin
  if EIcon <> Value then
  begin
    EIcon.Assign(Value);
    if (Owner is TForm) and (EIcon = TForm(Owner).Icon) then
      EIconType := itForm
    else if EIcon = Application.Icon then
      EIconType := itApplication
    else
      EIconType := itCustom;
  end;
end;

procedure TCustomTitlePanel.SetIconAlign(Value: TIconAlign);
begin
  if EIconAlign <> Value then
  begin
    EIconAlign := Value;
    Invalidate;
  end;
end;

procedure TCustomTitlePanel.SetBorder(Value: boolean);
begin
  EBorder := Value;
  Invalidate;
end;

procedure TCustomTitlePanel.SetIconType(Value: TIconType);
begin
  if (Value = itForm) and (Owner is TForm) and (not TForm(Owner).Icon.Empty) then
    EIcon.Assign(TForm(Owner).Icon)
  else if Value = itApplication then
    EIcon.Assign(Application.Icon)
  else if Value = itCustom then
    // Do nothing
  else
    Exit;
  EIconType := Value;
end;

procedure TCustomTitlePanel.SetRecord(Value: string);
begin
  if ERecord <> Value then
  begin
    ERecord := Value;
    Invalidate;
  end;
end;

procedure TCustomTitlePanel.SetRecFont(Value: TFont);
begin
  if ERecFont <> Value then
  begin
    ERecFont.Assign(Value);
    Invalidate;
  end;
end;

procedure TCustomTitlePanel.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
begin
  inherited SetBounds(Aleft,ATop,AWidth,AHeight);
  Invalidate;
end;

{ TSectionTitlePanel }

constructor TSectionTitlePanel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  EBitmap := TBitmap.Create;
  EBitmap.OnChange := BitmapChanged;
  EBitmap.TransparentMode := tmAuto;
  EBitmap.Transparent := True;
  Color := clBtnFace;
  Height := 20;
end;

destructor TSectionTitlePanel.Destroy;
begin
  EBitmap.Free;
  inherited;
end;

procedure TSectionTitlePanel.BitmapChanged(Sender: TObject);
begin
  Bitmap.Transparent := True;
  Invalidate;
end;

procedure TSectionTitlePanel.SetBitmap(Value: TBitmap);
begin
  if EBitmap <> Value then
    EBitmap.Assign(Value);
end;

procedure TSectionTitlePanel.Paint;
var
  R, BrdR1: TRect;
  y: Integer;
begin
  R := ClientRect;
  Canvas.Brush.Color := Self.Color;
  Canvas.FillRect(R);
  if Border then
  begin
    BrdR1 := ClientRect;
    Frame3D(Canvas, BrdR1, GetSysColor(COLOR_3DSHADOW), GetSysColor(COLOR_3DHILIGHT), 1);
  end;
  if not EBitmap.Empty then
  begin
    y := (Clientheight - EBitmap.height) div 2;
    Canvas.Draw(R.Right - EBitmap.Width - 8, y, EBitmap);
  end;
  R.Left := R.Left + 8;
  Canvas.Font.Assign(Self.Font);
  Canvas.Brush.Style := bsClear;
  DrawText(Canvas.Handle, PChar(Caption), Length(Caption), R, DT_SINGLELINE or DT_VCENTER);
end;

{ TSectionTitleBevel }

constructor TSectionTitleBevel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Color := clBtnFace;
  Height := 17;
  EIndent := 5;
end;

procedure TSectionTitleBevel.SetIndent(Value: integer);
begin
  if EIndent <> Value then
  begin
    EIndent := Value;
    Invalidate;
  end;
end;

procedure TSectionTitleBevel.Paint;
var
  R: TRect;
  Y: integer;
begin
  R := ClientRect;
  Canvas.Brush.Color := Self.Color;
  Canvas.FillRect(R);
  // Draw text
  R.Left := R.Left + EIndent;
  Canvas.Font.Assign(Self.Font);
  Canvas.Brush.Style := bsClear;
  DrawText(Canvas.Handle, PChar(Caption), Length(Caption), R, DT_SINGLELINE or DT_VCENTER);
  // Draw border
  R.Left := R.Left + Canvas.TextWidth(Caption) + 8;
  Y := (R.Bottom - R.Top) div 2;
  Canvas.Pen.Color := GetSysColor(COLOR_3DSHADOW);
  Canvas.MoveTo(R.Left,Y);
  Canvas.LineTo(R.Right, Y);
  Canvas.Pen.Color := GetSysColor(COLOR_3DHILIGHT);
  Canvas.MoveTo(R.Left, Y+1);
  Canvas.LineTo(R.Right, Y+1);
end;

{ TDialogTitlePanel }

constructor TDialogTitlePanel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Color := clWhite;
  Height := 60;
  ETitleLeft := 17;
  ETitleTop := 11;
  ECaptionLeft := 28;
  ECaptionTop := 32;
  ETitleFont := TFont.Create;
end;

procedure TDialogTitlePanel.Paint;
var
  R: TRect;
  y: Integer;
begin
  R := ClientRect;
  Canvas.Brush.Color := Self.Color;
  Canvas.FillRect(R);
  Y := R.Bottom -2;
  Canvas.Pen.Color := GetSysColor(COLOR_3DSHADOW);
  Canvas.MoveTo(R.Left,Y);
  Canvas.LineTo(R.Right, Y);
  Canvas.Pen.Color := GetSysColor(COLOR_3DHILIGHT);
  Canvas.MoveTo(R.Left, Y+1);
  Canvas.LineTo(R.Right, Y+1);
  if not EIcon.Empty then
  begin
    y := (Clientheight - EIcon.height) div 2;
    Canvas.Draw(R.Right - EIcon.Width -8, y, EIcon);
  end;

  Canvas.Font.Assign(Self.ETitleFont);
  Canvas.Brush.Style := bsClear;
  Canvas.TextOut( ETitleLeft, ETitleTop, ETitle );

  Canvas.Font.Assign(Self.Font);
  Canvas.Brush.Style := bsClear;
  Canvas.TextOut( ECaptionLeft, ECaptionTop, Caption );
end;

procedure TDialogTitlePanel.SetTitleFont(Value: TFont);
begin
  if ETitleFont <> Value then
  begin
    ETitleFont.Assign(Value);
    Invalidate;
  end;
end;

procedure TDialogTitlePanel.SetTitle(Value: string);
begin
  if ETitle <> Value then
  begin
    ETitle := Value;
    Invalidate;
  end;
end;

procedure TDialogTitlePanel.SetTitleLeft(Value: integer);
begin
  if ETitleLeft <> Value then
  begin
    ETitleLeft := Value;
    Invalidate;
  end;
end;

procedure TDialogTitlePanel.SetTitleTop(Value: integer);
begin
  if ETitleTop <> Value then
  begin
    ETitleTop := Value;
    Invalidate;
  end;
end;

procedure TDialogTitlePanel.SetCaptionLeft(Value: integer);
begin
  if ECaptionLeft <> Value then
  begin
    ECaptionLeft := Value;
    Invalidate;
  end;
end;

procedure TDialogTitlePanel.SetCaptionTop(Value: integer);
begin
  if ECaptionTop <> Value then
  begin
    ECaptionTop := Value;
    Invalidate;
  end;
end;

{ Procedures and Functions }

procedure Register;
begin
  RegisterComponents('PKH', [TTitlePanel, TSectionTitlePanel,
    TSectionTitleBevel, TDialogTitlePanel]);
end;

end.
